"use strict";

/* exported logger */
var Promise = require("bluebird");

var AbstractGuiElement = require('../AbstractGuiElement');
var AddOverlayDialog = require('../AddOverlayDialog');
var Annotation = require('../../map/data/Annotation');
var GuiConnector = require('../../GuiConnector');

var Functions = require('../../Functions');
var logger = require('../../logger');

var guiUtils = new (require('../leftPanel/GuiUtils'))();

function EditProjectDialog(params) {
    AbstractGuiElement.call(this, params);
    var self = this;
    $(self.getElement()).addClass("minerva-edit-project-dialog");

    self.createGui();
}

EditProjectDialog.prototype = Object.create(AbstractGuiElement.prototype);
EditProjectDialog.prototype.constructor = EditProjectDialog;

EditProjectDialog.prototype.createGui = function () {
    var self = this;
    var element = self.getElement();

    var tabDiv = Functions.createElement({
        type: "div",
        name: "tabView",
        className: "tabbable boxed parentTabs"
    });
    element.appendChild(tabDiv);

    var tabMenuDiv = Functions.createElement({
        type: "ul",
        className: "nav nav-tabs"
    });
    tabDiv.appendChild(tabMenuDiv);

    var tabContentDiv = Functions.createElement({
        type: "div",
        className: "tab-content"
    });
    tabDiv.appendChild(tabContentDiv);

    self.createGeneralTab(tabMenuDiv, tabContentDiv);
    self.createOverlaysTab(tabMenuDiv, tabContentDiv);
    self.createUsersTab(tabMenuDiv, tabContentDiv);
};

EditProjectDialog.prototype.createGeneralTab = function (tabMenuDiv, tabContentDiv) {
    var self = this;
    self.addTab({
        tabMenuDiv: tabMenuDiv,
        tabContentDiv: tabContentDiv,
        name: "GENERAL",
        id: self.getProject().getProjectId() + "_general_tab",
        content: self.createGeneralTabContent(),
    });

};

EditProjectDialog.prototype.addTab = function (params) {
    var navLi = guiUtils.createTabMenuObject({
        id: params.id,
        name: params.name,
        navigationBar: params.tabMenuDiv
    });
    params.tabMenuDiv.appendChild(navLi);

    var contentDiv = guiUtils.createTabContentObject({
        id: params.id,
        navigationObject: navLi,
        navigationBar: params.tabMenuDiv
    });

    if (params.content !== undefined) {
        contentDiv.appendChild(params.content);
    }

    params.tabContentDiv.appendChild(contentDiv);
};

EditProjectDialog.prototype.createGeneralTabContent = function () {
    var self = this;
    var project = self.getProject();

    var result = new Functions.createElement({
        type: "div",
    });

    var table = new Functions.createElement({
        type: "div",
        style: "display:table"
    });
    result.appendChild(table);

    var projectIdRow = new Functions.createElement({
        type: "div",
        style: "display:table-row"
    });
    table.appendChild(projectIdRow);
    projectIdRow.appendChild(new Functions.createElement({
        type: "div",
        style: "display:table-cell",
        content: "ProjectId",
    }));
    projectIdRow.appendChild(new Functions.createElement({
        type: "div",
        style: "display:table-cell",
        content: project.getProjectId(),
    }));

    var nameRow = new Functions.createElement({
        type: "div",
        style: "display:table-row"
    });
    table.appendChild(nameRow);
    nameRow.appendChild(new Functions.createElement({
        type: "div",
        style: "display:table-cell",
        content: "Name",
    }));
    nameRow.appendChild(new Functions.createElement({
        type: "div",
        style: "display:table-cell",
        content: "<input name='projectName' value='" + project.getName() + "'/>",
    }));

    var versionRow = new Functions.createElement({
        type: "div",
        style: "display:table-row"
    });
    table.appendChild(versionRow);
    versionRow.appendChild(new Functions.createElement({
        type: "div",
        style: "display:table-cell",
        content: "Version",
    }));
    versionRow.appendChild(new Functions.createElement({
        type: "div",
        style: "display:table-cell",
        content: "<input name='projectVersion' value='" + project.getVersion() + "'/>",
    }));

    var diseaseRow = new Functions.createElement({
        type: "div",
        style: "display:table-row"
    });
    table.appendChild(diseaseRow);
    diseaseRow.appendChild(new Functions.createElement({
        type: "div",
        style: "display:table-cell",
        content: "Disease",
    }));
    var disease = "";
    if (project.getDisease() !== undefined) {
        disease = project.getDisease().getResource();
    }
    diseaseRow.appendChild(new Functions.createElement({
        type: "div",
        style: "display:table-cell",
        content: "<input name='projectDisease' value='" + disease + "'/>",
    }));

    var organismRow = new Functions.createElement({
        type: "div",
        style: "display:table-row"
    });
    table.appendChild(organismRow);
    organismRow.appendChild(new Functions.createElement({
        type: "div",
        style: "display:table-cell",
        content: "Organism",
    }));
    var organism = "";
    if (project.getOrganism() !== undefined) {
        organism = project.getOrganism().getResource();
    }
    organismRow.appendChild(new Functions.createElement({
        type: "div",
        style: "display:table-cell",
        content: "<input name='projectOrganism' value='" + organism + "'/>",
    }));

    var emailRow = new Functions.createElement({
        type: "div",
        style: "display:table-row"
    });
    table.appendChild(emailRow);
    emailRow.appendChild(new Functions.createElement({
        type: "div",
        style: "display:table-cell",
        content: "Notify email",
    }));
    var email = "";
    if (project.getNotifyEmail() !== undefined) {
        email = project.getNotifyEmail();
    }
    emailRow.appendChild(new Functions.createElement({
        type: "div",
        style: "display:table-cell",
        content: "<input name='projectNotifyEmail' value='" + email + "'/>",
    }));

    var menuRow = Functions.createElement({
        type: "div",
        className: "minerva-menu-row",
        style: "display:table-row; margin:10px",
    });
    result.appendChild(menuRow);

    var saveProjectButton = Functions.createElement({
        type: "button",
        name: "saveProject",
        content: '<span class="ui-icon ui-icon-disk"></span>&nbsp;SAVE',
        onclick: function () {
            return self.onSaveClicked().then(function () {
                return self.close();
            }, GuiConnector.alert);
        },
    });
    var cancelButton = Functions.createElement({
        type: "button",
        name: "cancelProject",
        content: '<span class="ui-icon ui-icon-cancel"></span>&nbsp;CANCEL',
        onclick: function () {
            return self.close();
        },
    });
    menuRow.appendChild(saveProjectButton);
    menuRow.appendChild(cancelButton);

    return result;

};

EditProjectDialog.prototype.createOverlaysTab = function (tabMenuDiv, tabContentDiv) {
    var self = this;
    self.addTab({
        tabMenuDiv: tabMenuDiv,
        tabContentDiv: tabContentDiv,
        name: "OVERLAYS",
        id: self.getProject().getProjectId() + "_overlays_tab",
        content: self.createOverlaysTabContent(),
    });
};

EditProjectDialog.prototype.createOverlaysTabContent = function () {
    var self = this;
    var result = Functions.createElement({
        type: "div",
    });
    result.appendChild(self._createOverlayTable());
    return result;
};

EditProjectDialog.prototype._createOverlayTable = function () {
    var self = this;

    var result = Functions.createElement({
        type: "div",
        style: "margin-top:10px;",
    });

    var overlaysTable = Functions.createElement({
        type: "table",
        name: "overlaysTable",
        className: "display",
        style: "width:100%",
    });
    result.appendChild(overlaysTable);

    $(overlaysTable).DataTable({
        fnRowCallback: function (nRow, aData) {
            nRow.setAttribute('id', "overlay-" + aData[0]);
        },
        columns: [{
            title: 'Id',
        }, {
            title: 'Name',
        }, {
            title: 'Description',
        }, {
            title: 'Public',
        }, {
            title: 'Owner',
        }, {
            title: 'Data',
        }, {
            title: 'Update',
        }, {
            title: 'Remove',
        },],
        dom: '<"minerva-datatable-toolbar">frtip',
        initComplete: function () {
            $("div.minerva-datatable-toolbar", $(result)).html('<button name="addOverlay">Add overlay</button>');
        },

    });

    $(overlaysTable).on("click", "[name='removeOverlay']", function () {
        var button = this;
        return self.removeOverlay($(button).attr("data")).then(null, GuiConnector.alert);
    });

    $(overlaysTable).on("click", "[name='saveOverlay']", function () {
        var button = this;
        GuiConnector.showProcessing("Updating");
        return self.saveOverlay($(button).attr("data")).then(function () {
            GuiConnector.hideProcessing();
            GuiConnector.info("Overlay updated successfully");
        }, function (error) {
            GuiConnector.hideProcessing();
            GuiConnector.alert(error);
        });
    });

    $(overlaysTable).on("click", "[name='downloadSource']", function () {
        var button = this;
        return ServerConnector.getOverlaySourceDownloadUrl({
            overlayId: $(button).attr("data")
        }).then(function (url) {
            return self.downloadFile(url);
        }).then(null, GuiConnector.alert);
    });

    $(result).on("click", "[name='addOverlay']", function () {
        return self.openAddOverlayDialog();
    });

    return result;
};

EditProjectDialog.prototype.createUsersTab = function (tabMenuDiv, tabContentDiv) {
    var self = this;
    self.addTab({
        tabMenuDiv: tabMenuDiv,
        tabContentDiv: tabContentDiv,
        name: "USERS",
        id: self.getProject().getProjectId() + "_users_tab",
        content: self.createUsersTabContent(),
    });
};

EditProjectDialog.prototype.createUsersTabContent = function () {
    var self = this;

    var result = Functions.createElement({
        type: "div",
        style: "margin-top:10px;",
    });

    var usersTable = Functions.createElement({
        type: "table",
        name: "usersTable",
        className: "display",
        style: "width:100%",
    });
    result.appendChild(usersTable);

    $(usersTable).on("click", "[name='saveUser']", function () {
        var button = this;
        GuiConnector.showProcessing("Updating");
        return self.saveUser($(button).attr("data")).then(function () {
            GuiConnector.hideProcessing();
            GuiConnector.info("User updated successfully");
        }, function (error) {
            GuiConnector.hideProcessing();
            GuiConnector.alert(error);
        });
    });

    return result;
};

EditProjectDialog.prototype.createUserPrivilegeColumns = function () {
    var self = this;

    if (self._userPrivilegeColumns !== undefined) {
        return Promise.resolve(self._userPrivilegeColumns);
    }

    return ServerConnector.getConfiguration().then(function (configuration) {
        self._userPrivilegeColumns = [{
            title: "Name"
        }];
        var privilegeTypes = configuration.getPrivilegeTypes();
        for (var i = 0; i < privilegeTypes.length; i++) {
            var type = privilegeTypes[i];
            if (type.getObjectType() === "Project") {
                self._userPrivilegeColumns.push({
                    "title": type.getCommonName(),
                    privilegeType: type,
                });
            }
        }
        self._userPrivilegeColumns.push({
            "title": "Update"
        });
        return self._userPrivilegeColumns;
    });

};

EditProjectDialog.prototype.init = function () {
    var self = this;
    return self.initUsersTab().then(function () {
        return self.refreshUsers();
    }).then(function () {
        return self.refreshOverlays();
    }).then(function () {
        $(window).trigger('resize');
    });
};

EditProjectDialog.prototype.initUsersTab = function () {
    var self = this;

    var usersTable = $("[name=usersTable]", self.getElement())[0];

    return self.createUserPrivilegeColumns().then(function (columns) {
        $(usersTable).DataTable({
            columns: columns,
        });
    });
};

EditProjectDialog.prototype.refreshOverlays = function () {
    var self = this;
    return ServerConnector.getOverlays({
        projectId: self.getProject().getProjectId()
    }).then(function (overlays) {
        return self.setOverlays(overlays);
    });
};

EditProjectDialog.prototype.refreshUsers = function () {
    var self = this;
    return ServerConnector.getUsers().then(function (users) {
        return self.setUsers(users);
    });
};

EditProjectDialog.prototype.setOverlays = function (overlays) {
    var self = this;
    self._overlayById = [];
    return ServerConnector.getUsers().then(function (users) {
        var dataTable = $($("[name='overlaysTable']", self.getElement())[0]).DataTable();
        var data = [];
        for (var i = 0; i < overlays.length; i++) {
            var overlay = overlays[i];
            self._overlayById[overlay.getId()] = overlay;
            var rowData = self.overlayToTableRow(overlay, users);
            data.push(rowData);
        }
        dataTable.clear().rows.add(data).draw();
    });
};

EditProjectDialog.prototype.setUsers = function (users) {
    var self = this;
    self._userByLogin = [];
    return self.createUserPrivilegeColumns().then(function (columns) {
        var dataTable = $($("[name='usersTable']", self.getElement())[0]).DataTable();
        var data = [];
        for (var i = 0; i < users.length; i++) {
            var user = users[i];
            self._userByLogin[user.getLogin()] = user;
            var rowData = self.userToTableRow(user, columns);
            data.push(rowData);
        }
        dataTable.clear().rows.add(data).draw();
    });
};

EditProjectDialog.prototype.userToTableRow = function (user, columns) {
    var self = this;
    var row = [];
    var login = user.getLogin();

    row[0] = user.getName() + " " + user.getSurname() + " (" + login + ")";
    for (var i = 1; i < columns.length; i++) {
        var column = columns[i];
        if (column.privilegeType !== undefined) {
            if (column.privilegeType.getValueType() === "boolean") {
                var checked = '';
                if (user.hasPrivilege(column.privilegeType, self.getProject().getId())) {
                    checked = 'checked';
                }
                row[i] = "<input type='checkbox' name='privilege-" + login + "' data='" + column.privilegeType.getName() + "' "
                    + checked + "/>";
            } else {
                throw new Error("Unsupported type: " + column.privilegeType.getValueType());
            }
        }
    }

    row.push("<button name='saveUser' data='" + login + "'>SAVE</button>");

    return row;
};

EditProjectDialog.prototype.overlayToTableRow = function (overlay, users) {
    var row = [];
    var id = overlay.getId();
    var creatorSelect;
    if (overlay.getCreator() === "") {
        creatorSelect = "<select name='creator-" + id + "' value=''>";
    } else {
        creatorSelect = "<select name='creator-" + id + "'>";
    }

    creatorSelect += "<option value='' >---</option>";
    for (var i = 0; i < users.length; i++) {
        var selected = "";
        var user = users[i];
        if (overlay.getCreator() === user.getLogin()) {
            selected = "selected";
        } else {
            selected = "";
        }

        creatorSelect += "<option value='" + user.getLogin() + "' " + selected + ">" + user.getLogin() + "("
            + user.getName() + " " + user.getSurname() + ")</option>";
    }
    creatorSelect += "</select>";

    var checked = '';
    if (overlay.getPublicOverlay()) {
        checked = "checked";
    }
    var publicOverlayCheckbox = "<input type='checkbox' name='publicOverlay-" + id + "' " + checked + "/>";

    var downloadSourceButton;
    if (overlay.getInputDataAvailable()) {
        downloadSourceButton = "<button name='downloadSource' data='" + id + "'>"
            + "<span class='ui-icon ui-icon-arrowthickstop-1-s'></span>" + "</button>";
    } else {
        downloadSourceButton = "N/A";
    }

    row[0] = id;
    row[1] = "<input name='name-" + id + "' value='" + overlay.getName() + "'/>";
    row[2] = "<input name='description-" + id + "' value='" + overlay.getDescription() + "'/>";
    row[3] = publicOverlayCheckbox;
    row[4] = creatorSelect;
    row[5] = downloadSourceButton;
    row[6] = "<button name='saveOverlay' data='" + id + "'>SAVE</button>";
    row[7] = "<button name='removeOverlay' data='" + id + "'>REMOVE</button>";

    return row;
};

EditProjectDialog.prototype.destroy = function () {
    var self = this;
    var div = self.getElement();
    var usersTable = $("[name=usersTable]", self.getElement())[0];
    var overlaysTable = $("[name=overlaysTable]", self.getElement())[0];
    if ($.fn.DataTable.isDataTable(usersTable)) {
        $(usersTable).DataTable().destroy();
    }
    if ($.fn.DataTable.isDataTable(overlaysTable)) {
        $(overlaysTable).DataTable().destroy();
    }

    if ($(div).hasClass("ui-dialog-content")) {
        $(div).dialog("destroy");
    }
    if (self._addOverlayDialog !== undefined) {
        self._addOverlayDialog.destroy();
        delete self._addOverlayDialog;
    }
};

EditProjectDialog.prototype.open = function () {
    var self = this;
    var div = self.getElement();
    if (!$(div).hasClass("ui-dialog-content")) {
        $(div).dialog({
            title: self.getProject().getProjectId(),
            width: window.innerWidth / 2,
            height: window.innerHeight / 2,
        });
    }
    $(div).dialog("open");
};

var prepareMiriamData = function (type, resource) {
    if (resource === "" || resource === undefined || resource === null) {
        return null;
    } else {
        return new Annotation({
            type: type,
            resource: resource
        });
    }
};

EditProjectDialog.prototype.onSaveClicked = function () {
    var self = this;
    var project = self.getProject();
    var element = self.getElement();
    project.setName($("[name='projectName']", element)[0].value);
    project.setVersion($("[name='projectVersion']", element)[0].value);
    project.setNotifyEmail($("[name='projectNotifyEmail']", element)[0].value);
    var organism = prepareMiriamData("TAXONOMY", $("[name='projectOrganism']", element)[0].value);
    project.setOrganism(organism);
    var disease = prepareMiriamData("MESH_2012", $("[name='projectDisease']", element)[0].value);
    project.setDisease(disease);
    return ServerConnector.updateProject(project);
};
EditProjectDialog.prototype.close = function () {
    var self = this;
    $(self.getElement()).dialog("close");
};

EditProjectDialog.prototype.saveOverlay = function (overlayId) {
    var self = this;
    var overlay = self._overlayById[overlayId];
    overlay.setName($("[name='name-" + overlayId + "']", self.getElement())[0].value);
    overlay.setDescription($("[name='description-" + overlayId + "']", self.getElement())[0].value);
    overlay.setPublicOverlay($("[name='publicOverlay-" + overlayId + "']", self.getElement())[0].checked);
    overlay.setCreator($("[name='creator-" + overlayId + "']", self.getElement())[0].value);

    return ServerConnector.updateOverlay(overlay);
};

EditProjectDialog.prototype.saveUser = function (login) {
    var self = this;
    var checkboxes = $("[name='privilege-" + login + "']", self.getElement());
    var privileges = {};
    for (var i = 0; i < checkboxes.length; i++) {
        var checkbox = checkboxes[i];
        var privilege = {};
        privilege[self.getProject().getId()] = checkbox.checked;
        privileges[$(checkbox).attr("data")] = privilege;
    }

    return ServerConnector.updateUserPrivileges({
        user: self._userByLogin[login],
        privileges: privileges
    });
};

EditProjectDialog.prototype.removeOverlay = function (overlayId) {
    var self = this;
    return ServerConnector.removeOverlay({
        overlayId: overlayId
    }).then(function () {
        return self.refreshOverlays();
    });
};

EditProjectDialog.prototype.openAddOverlayDialog = function () {
    var self = this;
    if (self._addOverlayDialog !== undefined) {
        self._addOverlayDialog.destroy();
    }
    self._addOverlayDialog = new AddOverlayDialog({
        project: self.getProject(),
        customMap: null,
        element: document.createElement("div"),
    });
    self._addOverlayDialog.addListener("onAddOverlay", function () {
        return self.refreshOverlays();
    });
    return self._addOverlayDialog.init().then(function () {
        return self._addOverlayDialog.open();
    });
};

module.exports = EditProjectDialog;
